/**
 * openbox-xdgmenu
 *
 * This command line application generates an OpenBox menu XML segment based on
 * an Xdg menu structure.
 *
 * Usage: openbox-xdgmenu <Xdg menu file>
 *
 * Copyright (C) Nathan Fisher <nfisher@grafpup.com>
 * Copyright (C) 2008 Siegfried-A. Gevatter <rainct@ubuntu.com>
 * Copyright (C) 2013 J. M. Becker <zilla@techzilla.info>
 * Originally based upon code by Raul Suarez <rarsa@yahoo.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Dependencies:
 *   gnome-menus
 *   glib
 *   gtk
 */

#include <string.h>
#include <glib.h>
#include <glib/gprintf.h>
#include <gnome-menus/gmenu-tree.h>
#include <getopt.h>
#include <gtk/gtk.h>

static void show_help();
static void process_directory(GMenuTreeDirectory *directory, gboolean isRoot);
static void process_entry(GMenuTreeEntry *entry);
static void process_separator(GMenuTreeSeparator *entry);

gboolean option_show_amount_of_entries = 0;
gboolean option_do_not_use_categories = 0;

void *xdgfile;
GtkIconTheme *icon_theme;


/*=============================================================================
 * Main Function
 */

int
main (int argc, char *argv[])
{
    int c;

    while (1) {
        static struct option long_options[] = {
            {"show-amount", no_argument,       0, 'a'},
            {"help",  no_argument,       0, 'h'},
            {"plain",   no_argument,       0, 'p'},
            {0, 0, 0, 0}
        };

        int option_index = 0;
        c = getopt_long (argc, argv, "ahp",
                         long_options, &option_index);

        if (c == -1)
            break;

        switch (c) {

        case 'a':
            option_show_amount_of_entries = 1;
            break;

        case 'h':
            show_help ();
            return 1;

        case 'p':
            option_do_not_use_categories = 1;
            break;

        case '?':
        default:
            return 1;
        }
    }

    if (++optind != argc) {
        show_help ();
        g_printf ("Unexpected or missing argunments");
        return 1;
    }

    gchar *xdgfile = g_strdup (argv[--optind]);

    FILE * file;
    if (!(file = fopen(xdgfile, "r"))) {
        g_printf ("Could not read file \"%s\".\n", xdgfile);
        g_free (xdgfile);
        return 1;
    }

    fclose(file);

    gtk_init (&argc, &argv);
    icon_theme = gtk_icon_theme_get_default ();

    GMenuTree *menuTree = gmenu_tree_lookup (xdgfile,  GMENU_TREE_FLAGS_NONE );

    GMenuTreeDirectory *rootDirectory = gmenu_tree_get_root_directory(menuTree);

    g_printf ("<?xml version=\"1.0\"?>\n");
    g_printf ("<openbox_pipe_menu>\n");

    process_directory(rootDirectory, 1);

    gmenu_tree_item_unref (rootDirectory);

    g_printf ("</openbox_pipe_menu>\n");

    g_free (xdgfile);
    return 0;
}

/*=============================================================================
 * Shows usage parameters
 */
void show_help()
{
    g_printf ("Creates an OpenBox menu from an Xdg menu structure.\n");
    g_printf ("\n");
    g_printf ("Usage:\n");
    g_printf ("  openbox-xdgmenu [options] <Xdg menu file>\n");
    g_printf ("\n");
    g_printf ("Options:\n");
    g_printf ("  -a: Show the amount of items in each category next to its name.\n");
    g_printf ("  -p: Do not use categories.\n");
    g_printf ("\n");
    g_printf ("For example:\n");
    g_printf ("  openbox-xdgmenu \"/etc/xdg/menus/applications.menu\"\n\n");
}

/*=============================================================================
 * This function processes a directory entry and all it's child nodes
 */
void process_directory(GMenuTreeDirectory *directory, gboolean isRoot)
{
    gboolean hasSeparator = 0;
    gboolean hasMenu = 0;
    GMenuTreeItemType entryType;
    GSList *entryList = gmenu_tree_directory_get_contents (directory);
    GSList *l;
    const gchar *empty = "empty";

    gchar *icon_name = g_strdup(gmenu_tree_directory_get_icon(directory));


    if (!g_path_is_absolute(icon_name)) {

        if (!icon_name) {
            icon_name = g_strdup(empty);
        }

        GtkIconInfo *icon_info = gtk_icon_theme_lookup_icon (icon_theme, icon_name, 22, GTK_ICON_LOOKUP_NO_SVG | GTK_ICON_LOOKUP_GENERIC_FALLBACK);
        icon_name = g_strdup(gtk_icon_info_get_filename(icon_info));
        g_object_unref(icon_info);
    }


    if (option_do_not_use_categories == 0 && isRoot == 0 &&
        g_slist_length(entryList) > 0) {
        hasMenu = 1;

        if (option_show_amount_of_entries == 1) {
            g_printf(
                "<menu id=\"xdgmenu-%s\" label=\"%s (%d)\" icon=\"%s\">\n",
                g_strjoinv("&amp;", g_strsplit(gmenu_tree_directory_get_name(directory),"&",0)),
                g_strjoinv("&amp;", g_strsplit(gmenu_tree_directory_get_name(directory),"&",0)),
                g_slist_length(entryList),
                icon_name);
        } else {
            g_printf(
                "<menu id=\"xdgmenu-%s\" label=\"%s\" icon=\"%s\">\n",
                g_strjoinv("&amp;", g_strsplit(gmenu_tree_directory_get_name(directory),"&",0)),
                g_strjoinv("&amp;", g_strsplit(gmenu_tree_directory_get_name(directory),"&",0)),
                icon_name);
        }
    }
    g_free(icon_name);

    for (l = entryList; l; l = l->next) {
        GMenuTreeItem *item = l->data;

        entryType = gmenu_tree_item_get_type (GMENU_TREE_ITEM(item));

        switch (entryType) {
        case GMENU_TREE_ITEM_DIRECTORY:
            if (hasSeparator) {
                process_separator(GMENU_TREE_SEPARATOR(item));
                hasSeparator = 0;
            }
            process_directory(GMENU_TREE_DIRECTORY(item), 0);
            break;
        case GMENU_TREE_ITEM_ENTRY:
            if (hasSeparator) {
                process_separator(GMENU_TREE_SEPARATOR(item));
                hasSeparator = 0;
            }
            process_entry(GMENU_TREE_ENTRY(item));
            break;
        case GMENU_TREE_ITEM_SEPARATOR:
            hasSeparator = 1;
            break;
        }

        gmenu_tree_item_unref (item);
    }

    if (hasMenu == 1) {
        g_printf("</menu>\n");
    }

    g_slist_free (entryList);
}


/*=============================================================================
 * This function adds an application entry
 */
void process_entry(GMenuTreeEntry *entry)
{
    gchar *name = g_strdup (gmenu_tree_entry_get_name(entry));
    gchar *exec = g_strdup (gmenu_tree_entry_get_exec(entry));
    gchar *icon_name = g_strdup(gmenu_tree_entry_get_icon(entry));
    const gchar *empty = "empty";

    if (!g_path_is_absolute(icon_name)) {

        if (!icon_name) {
            icon_name = g_strdup(empty);
        }

        GtkIconInfo *icon_info = gtk_icon_theme_lookup_icon (icon_theme, icon_name, 22, GTK_ICON_LOOKUP_NO_SVG | GTK_ICON_LOOKUP_GENERIC_FALLBACK);
        icon_name = g_strdup(gtk_icon_info_get_filename(icon_info));
        g_object_unref(icon_info);
    }

    int i;

    for (i = 0; i < strlen(exec) - 1; i++) {
        if (exec[i] == '%') {
            switch (exec[i+1]) {
            case 'f':
            case 'F':
            case 'u':
            case 'U':
            case 'd':
            case 'D':
            case 'n':
            case 'N':
            case 'i':
            case 'c':
            case 'k':
            case 'v':
            case 'm':
                exec[i] = '\0';
                i++;
                break;
            }
        }
    }

    g_printf("<item label=\"%s\" icon=\"%s\">\n",
             g_strjoinv("&amp;", g_strsplit(name,"&",0)), icon_name);

    g_printf("<action name=\"Execute\"><command>%s</command></action>\n</item>\n", exec);

    g_free(name);
    g_free(exec);
    g_free(icon_name);
}

/*=============================================================================
 * This function adds a separator
 */
void process_separator(GMenuTreeSeparator *entry)
{
    g_printf("<separator /> \n");
}

/*=============================================================================
 */
